#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <arpa/inet.h>

#include "chksum.h"

/* See http://www.netfor2.com/checksum.html */
uint16_t chksum (void * buffer, uint16_t bytes)
{
  uint16_t * words = (uint16_t *) buffer;
  uint8_t wc = (uint8_t) (bytes / 2);
  uint32_t sum = 0;
  uint8_t i;

#ifdef DEBUG_CHKSUM
  fprintf(stderr, "[chksum] bytes = %d\n", bytes);
  fprintf(stderr, "[chksum] words = %d\n", wc);
#endif
 
  for (i = 0 ; i < wc ; i++)
    {
#ifdef DEBUG_CHKSUM
      fprintf(stderr, "[chksum] sum (%#08x) + word[%2d] (0x%04x) = new sum (%#08x)\n",
	      sum, i, words[i], sum + words[i]);
#endif
      sum += words[i];
    }

  /* Catches odd bytes. This should not happen;
     IP headers are supposed to be padded. */
  if (bytes % 2 == 1)
    {
#ifdef DEBUG_CHKSUM
      fprintf(stderr, "[chksum] sum (%#08x) + odd byte (0x%04x) = new sum (%#08x)\n",
	      sum, words[wc] & 0x00FF, sum + (words[wc] & 0x00FF));
#endif
      sum += words[wc] & 0x00FF;
    }
 
  /* Even a massive packet of 65535 bytes can only produce
     two bytes of overflow with all words set to 0xFFFF:
     (2^16-1) * (2^16-1) < 2^32. */
#ifdef DEBUG_CHKSUM
  fprintf(stderr,
	  "[chksum] sum (%#04x) + overflow (%#04x) = one's complement sum (%#04x)\n",
	  sum & 0xFFFF, (sum >> 16 & 0xFFFF), (sum >> 16) + (sum & 0xFFFF));
#endif
  sum = (sum >> 16) + (sum & 0xFFFF);


#ifdef DEBUG_CHKSUM
  fprintf(stderr, "[chksum] One's complement of one's complement sum = %#04x\n", ~sum);
#endif
  return ~sum;
}


