#define _GNU_SOURCE // required for NI_MAXHOST in netdb.h

#include "getaddr.h"
#include <stdio.h>
#include <ifaddrs.h>
#include <stdlib.h>
#include <netdb.h>
#include <unistd.h>
#include <linux/if_link.h>
#include <inttypes.h>
#include <string.h>
#include <sys/socket.h>

static char iface[16];

/* https://trac.transmissionbt.com/browser/trunk/third-party/libnatpmp/getgateway.c */
char * getDefaultInterface ()
{
  char line[1024], interface[16];
  uint64_t destination, gateway, mask, flags;
  int refcnt, use, metric;
  FILE * f;
  int lines = 0;

  if ((f = fopen("/proc/net/route", "r")) == NULL)
    {
      perror("fopen");
      exit(EXIT_FAILURE);
    }

#ifdef DEBUG_GETADDR
  printf("[getaddr] Iface\t\tDestination\tGateway\t\tFlags\tRefCnt\tUse\tMetric\tMask\n");
#endif

  while (fgets(line, 1024, f))
    {
      if (lines > 0 && strnlen(line, 1024) > 0)
	{
	  sscanf(line, "%s\t%lx%lx%lx%d%d%d%lx", interface, &destination, &gateway, &flags, &refcnt, &use, &metric, &mask);

#ifdef DEBUG_GETADDR
	  printf("[getaddr] %s\t%08lX\t%08lX\t%04lX\t%d\t%d\t%d\t%08lX\n", interface, destination, gateway, flags, refcnt, use, metric, mask);
#endif

	  if (destination == 0 && mask == 0)
	    strncpy(iface, interface, 16);
	}
      lines++;
    }

  if (fclose(f) != 0)
    {
      perror("fclose");
      exit(EXIT_FAILURE);
    }

#ifdef DEBUG_GETADDR
  printf("[getaddr] Found default interface: %s\n", iface);
#endif

  return iface;
}

uint32_t getAddress (char * iface)
{
  /* Base address of first interface */
  struct ifaddrs * ifaddr;

  /* Current interface */
  struct ifaddrs * ifa;

  /* Address socket address structure */
  struct sockaddr_in * sa;

  uint32_t address = 0;

  if (getifaddrs(&ifaddr) == -1)
    {
      perror("getifaddrs");
      exit(EXIT_FAILURE);
    }

  for (ifa = ifaddr ; ifa != NULL && address == 0 ; ifa = ifa->ifa_next)
    {
      if (ifa->ifa_addr == NULL)
	continue;

      if (ifa->ifa_addr->sa_family == AF_INET && (strncmp(ifa->ifa_name, iface, 16) == 0))
	{
	  sa = (struct sockaddr_in *) ifa->ifa_addr;
	  address = (uint32_t) sa->sin_addr.s_addr;
	}
    }

  freeifaddrs(ifaddr);

  if (address == 0)
    {
      fprintf(stderr, "[getaddr] Interface %s not found.\n", iface);
      exit(EXIT_FAILURE);
    }
  return address;
}

uint32_t getNetmask (char * iface)
{
  /* Base address of first interface */
  struct ifaddrs * ifaddr;

  /* Current interface */
  struct ifaddrs * ifa;

  /* Netmask socket address structure */
  struct sockaddr_in * nm;

  uint32_t netmask = 0;

  if (getifaddrs(&ifaddr) == -1)
    {
      perror("getifaddrs");
      exit(EXIT_FAILURE);
    }

  for (ifa = ifaddr ; ifa != NULL && netmask == 0 ; ifa = ifa->ifa_next)
    {
      if (ifa->ifa_addr == NULL)
	continue;

      if (ifa->ifa_addr->sa_family == AF_INET && (strncmp(ifa->ifa_name, iface, 16) == 0))
	{
	  nm = (struct sockaddr_in *) ifa->ifa_netmask;
	  netmask = (uint32_t) nm->sin_addr.s_addr;
	}
    }

  freeifaddrs(ifaddr);

  if (netmask == 0)
    {
      fprintf(stderr, "[getaddr] Interface %s not found.\n", iface);
      exit(EXIT_FAILURE);
    }
  return netmask;
}
